@@ -28,7 +28,7 @@ from dvhaedit.threads import ParsingProgressFrame, SavingProgressFrame, RefSyncP
 from dvhaedit.dicom_editor import Tag
 from dvhaedit.dynamic_value import ValueGenerator
 from dvhaedit.options import Options
-from dvhaedit.utilities import set_msw_background_color, get_file_paths, get_type, get_selected_listctrl_items,\
+from dvhaedit.utilities import set_msw_background_color, get_file_paths, get_selected_listctrl_items,\
     is_mac, save_object_to_file, load_object_from_file, set_frame_icon
 
 
@@ -53,7 +53,6 @@ class MainFrame(wx.Frame):
         self.input = {key: wx.TextCtrl(self, wx.ID_ANY, "") for key in keys}
         self.input['preview'] = wx.TextCtrl(self, wx.ID_ANY, "", style=wx.TE_READONLY)
         self.input['selected_file'] = wx.ComboBox(self, wx.ID_ANY, style=wx.CB_DROPDOWN | wx.CB_READONLY)
-        self.input['value_type'] = wx.ComboBox(self, wx.ID_ANY, style=wx.CB_DROPDOWN | wx.CB_READONLY)
         self.input_text_obj = [self.input[key] for key in keys]  # use for text event binding and focusing
 
         keys = ['save_dicom', 'quit', 'in_browse', 'out_browse', 'add', 'delete', 'select_all', 'deselect_all',
@@ -64,12 +63,12 @@ class MainFrame(wx.Frame):
         bmp = wx.ArtProvider.GetBitmap(wx.ART_INFORMATION, size=(16, 16))
         self.button['value_help'] = wx.BitmapButton(self, id=wx.ID_ANY, bitmap=bmp)
 
-        columns = ['Index', 'Tag', 'Keyword', 'Value', 'Value Type']
+        columns = ['Tag', 'Keyword', 'Value']
         data = {c: [''] for c in columns}
         self.list_ctrl = wx.ListCtrl(self, wx.ID_ANY, style=wx.BORDER_SUNKEN | wx.LC_REPORT)
-        self.data_table = DataTable(self.list_ctrl, data=data, columns=columns, widths=[-2] * 5)
+        self.data_table = DataTable(self.list_ctrl, data=data, columns=columns, widths=[-2] * len(columns))
 
-        keys = ['tag_group', 'tag_element', 'value', 'value_type', 'files_found', 'keyword', 'selected_file',
+        keys = ['tag_group', 'tag_element', 'value', 'files_found', 'keyword', 'selected_file',
                 'modality', 'prepend_file_name', 'search', 'value_rep', 'preview', 'advanced', 'value_help']
         self.label = {key: wx.StaticText(self, wx.ID_ANY, key.replace('_', ' ').title() + ':') for key in keys}
 
@@ -140,9 +139,6 @@ class MainFrame(wx.Frame):
         self.input['selected_file'].SetToolTip(msg)
         self.label['selected_file'].SetToolTip(msg)
 
-        self.input['value_type'].SetItems(['str', 'float', 'int'])
-        self.input['value_type'].SetValue('str')
-
         self.update_referenced_tags.SetValue(self.referenced_tag_choices[1])
         self.update_referenced_tags.SetToolTip("Automatically sync Referenced<tag> to new <tag> value to maintain "
                                                "cross-file connections (e.g., keep RT-Structure connection to "
@@ -206,6 +202,7 @@ class MainFrame(wx.Frame):
         pub.subscribe(self.call_next_value_generator, 'value_gen_complete')
         pub.subscribe(self.update_dicom_edits, 'update_dicom_edits')
         pub.subscribe(self.do_save_dicom, 'do_save_dicom')
+        pub.subscribe(self.do_save_dicom_step_2, 'do_save_dicom_step_2')
 
     def __do_layout(self):
         """Create GUI layout"""
@@ -218,7 +215,7 @@ class MainFrame(wx.Frame):
         sizer_edit_wrapper = wx.StaticBoxSizer(wx.StaticBox(self, wx.ID_ANY, "Tag Editor"), wx.VERTICAL)
         sizer_edit = wx.BoxSizer(wx.HORIZONTAL)
         sizer_edit_widgets = {key: wx.BoxSizer(wx.VERTICAL)
-                              for key in ['tag_group', 'tag_element', 'value', 'value_type', 'value_help',
+                              for key in ['tag_group', 'tag_element', 'value', 'value_help',
                                           'add', 'search', 'advanced']}
         sizer_value_keyword = wx.BoxSizer(wx.VERTICAL)
         sizer_edit_buttons = wx.BoxSizer(wx.HORIZONTAL)
@@ -245,7 +242,7 @@ class MainFrame(wx.Frame):
         sizer_main.Add(sizer_input_dir_wrapper, 0, wx.EXPAND | wx.ALL, 5)
 
         # Input Widgets
-        for key in ['search', 'tag_group', 'tag_element', 'value_type', 'value_help', 'advanced']:
+        for key in ['search', 'tag_group', 'tag_element', 'value_help', 'advanced']:
             obj = self.button if key in {'search', 'value_help', 'advanced'} else self.input
             sizer_edit_widgets[key].Add(self.label[key], 0, wx.EXPAND, 0)
             sizer_edit_widgets[key].Add(obj[key], 0, wx.EXPAND, 0)
@@ -321,11 +318,7 @@ class MainFrame(wx.Frame):
 
     @property
     def value(self):
-        value = self.input['value'].GetValue()
-        type_ = get_type(self.input['value_type'].GetValue())
-        if len(value) and value[0] == '[' and value[-1] == ']':
-            return value
-        return type_(value)
+        return self.input['value'].GetValue()
 
     @property
     def keyword(self):
@@ -390,14 +383,19 @@ class MainFrame(wx.Frame):
 
     def on_add(self, *evt):
         """Add a tag edit"""
-        keys_int = [int(key) for key in list(self.all_options)]
-        row_index = '0' if not keys_int else str(max(keys_int) + 1)
-        self.all_options[row_index] = deepcopy(self.current_options)
-        row = [row_index,
-               str(self.tag),
-               self.keyword,
-               self.input['value'].GetValue(),
-               self.input['value_type'].GetValue()]
+
+        tag_str = str(self.tag)
+
+        # Delete tag in table if found
+        row_index = self.data_table.get_row_index_from_tag(tag_str)
+        if row_index is not None:
+            self.data_table.delete_row(row_index)
+
+        # Store options
+        self.all_options[tag_str] = deepcopy(self.current_options)
+
+        # Add data to table
+        row = [str(self.tag), self.keyword, self.input['value'].GetValue()]
         if self.data_table_has_data:
             self.data_table.append_row(row)
         else:
@@ -406,6 +404,7 @@ class MainFrame(wx.Frame):
             self.data_table.set_data(data, columns)
         self.data_table.set_column_widths(auto=True)
 
+        # Reset input widgets
         for key in ['tag_group', 'tag_element', 'value']:
             self.input[key].SetValue('')
 
@@ -428,10 +427,14 @@ class MainFrame(wx.Frame):
     def on_delete(self, *evt):
         """Delete the selected tag edits"""
         for index in self.selected_indices[::-1]:
+            tag = self.data_table.get_row(index)[0]
+            if tag in list(self.all_options):
+                self.all_options.pop(tag)
             self.data_table.delete_row(index)
         self.update_delete_enable()
         self.update_save_template_enable()
         self.update_save_dicom_enable()
+        self.update_preview()
 
     def on_select_all(self, *evt):
         """Select all tag edits"""
@@ -487,12 +490,11 @@ class MainFrame(wx.Frame):
         self.update_delete_enable()
         selected_data = self.data_table.selected_row_data
         if selected_data:
-            tag = selected_data[0][1][1:-1].split(',')
+            tag = selected_data[0][0][1:-1].split(',')
             group = tag[0].strip()
             element = tag[1].strip()
             self.input['tag_group'].SetValue(group)
             self.input['tag_element'].SetValue(element)
-            self.input['value_type'].SetValue(selected_data[0][4])
             self.update_keyword()
             self.update_init_value()
             self.update_preview()
@@ -657,20 +659,8 @@ class MainFrame(wx.Frame):
         """Update Keyword in the Tag Editor based on the current Tag and currently selected file"""
         keyword = self.keyword if self.group and self.element else ''
         self.label['keyword'].SetLabel("Keyword: %s" % keyword)
-        self.update_tag_type()
         self.update_vr()
-
-    def update_tag_type(self):
-        """Update tag type in the Tag Editor based on the current Tag and currently selected file"""
-        tag = self.tag.tag
-        tag_type = 'str'
-        for file_path in self.file_paths:
-            try:
-                tag_type = self.ds[file_path].get_tag_type(tag)
-                break
-            except Exception:
-                pass
-        self.input['value_type'].SetValue(tag_type)
+        self.update_add_button_label()
 
     def update_vr(self):
         value = self.tag.vr if self.group and self.element else ''
@@ -678,8 +668,6 @@ class MainFrame(wx.Frame):
 
     def update_preview(self):
         """Apply the tag edits to every file in self.ds, return any errors"""
-        # TODO: auto-update Value Type based on VR
-        # TODO: validate New Value against Value Type
         tag = self.tag
         value_str = self.value
         value_gen = ValueGenerator(value_str, tag.tag, self.current_options)
@@ -687,6 +675,11 @@ class MainFrame(wx.Frame):
         values = value_gen(self.ds, file_path=file) if file in self.ds.keys() else ''
         value = str(values[0]) if values else ''
         self.input['preview'].SetValue(value)
+        self.update_add_button_label()
+
+    def update_add_button_label(self):
+        label = 'Update' if str(self.tag) in list(self.all_options) else 'Add'
+        self.button['add'].SetLabel(label)
 
     #################################################################################
     # Data updaters
@@ -770,12 +763,11 @@ class MainFrame(wx.Frame):
 
     def get_table_row_data(self, row):
         row_data = self.data_table.get_row(row)
-        group = row_data[1].split(',')[0][1:].strip()
-        element = row_data[1].split(',')[1][:-1].strip()
+        group = row_data[0].split(',')[0][1:].strip()
+        element = row_data[0].split(',')[1][:-1].strip()
         return {'tag': Tag(group, element),
-                'keyword': row_data[2],
-                'value_str': row_data[3],
-                'value_type': get_type(row_data[4]),
+                'keyword': row_data[1],
+                'value_str': row_data[2],
                 'options': self.all_options[row_data[0]]}
 
     def set_output_paths(self, check_only=False):
@@ -856,7 +848,7 @@ class MainFrame(wx.Frame):
         if self.value_generators:
             value_generator = self.value_generators.pop(0)
             iteration = self.data_table.row_count - len(self.value_generators)
-            ValueGenProgressFrame(self.ds, value_generator, iteration, self.data_table.row_count)
+            wx.CallAfter(ValueGenProgressFrame, self.ds, value_generator, iteration, self.data_table.row_count)
         else:
             wx.CallAfter(self.apply_edits)
 
@@ -866,7 +858,7 @@ class MainFrame(wx.Frame):
     def apply_edits(self):
         """Send table data to apply edits thread, edit DICOM tags"""
         all_row_data = [self.get_table_row_data(row) for row in range(self.data_table.row_count)]
-        ApplyEditsProgressFrame(self.ds, self.values_dicts, all_row_data)
+        wx.CallAfter(ApplyEditsProgressFrame, self.ds, self.values_dicts, all_row_data)
 
     # ------------------------------------------------------------------------------
     # Step 3: View error log, check for file over-writing, sync referenced tags, save DICOM to file
@@ -874,11 +866,18 @@ class MainFrame(wx.Frame):
     def do_save_dicom(self):
         # If any errors detected, display in GUI and ask user if they'd like to continue
         if self.error_log:
-            ViewErrorLog(self.error_log)
-            with AskYesNo(self, "Ignore errors and write DICOM files anyway?") as dlg:
-                if dlg.ShowModal() == wx.ID_NO:
-                    return
+            wx.CallAfter(ViewErrorLog, self.error_log)
+        else:
+            wx.CallAfter(self.do_save_dicom_step_2)
 
+    def do_save_dicom_step_1a(self):
+        with AskYesNo(self, "Ignore errors and write DICOM files anyway?") as dlg:
+            if dlg.ShowModal() == wx.ID_NO:
+                return
+            else:
+                wx.CallAfter(self.do_save_dicom_step_2)
+
+    def do_save_dicom_step_2(self):
         # set_output_paths(check_only=True) will return True if any file would be over-written
         if self.set_output_paths(check_only=True):
             msg = "You will overwrite files with this action. Continue?"
@@ -887,6 +886,10 @@ class MainFrame(wx.Frame):
             with wx.MessageDialog(self, msg, caption, flags) as dlg:
                 if dlg.ShowModal() == wx.ID_NO:
                     return
+
+        wx.CallAfter(self.do_save_dicom_step_3)
+
+    def do_save_dicom_step_3(self):
         self.set_output_paths()
 
         # Check for any referenced tags by sending edit history to each data set
@@ -898,7 +901,7 @@ class MainFrame(wx.Frame):
             self.do_saving_progress_frame()
 
     def do_saving_progress_frame(self):
-        SavingProgressFrame(self.ds.values())
+        wx.CallAfter(SavingProgressFrame, self.ds.values())
 
     # ------------------------------------------------------------------------------
     # Step 4: Save history, if requested, reparse original data since it has been edited directly
